@@ -1,3 +1,4 @@
+//@line 39 "/builds/tinderbox/Fx-Mozilla1.8.0/Linux_2.4.21-37.ELsmp_Depend/mozilla/toolkit/content/fontpackage.js"
 
 var gLangCode;
 
@@ -17,8 +18,7 @@ function onLoad()
     // if no download button
     // set title to "Install Font"
     // and set cancel button to "OK"
-    var downloadFontDialog = document.getElementById("downloadFontDialog");
-    downloadFontDialog.setAttribute("title", fontPackageBundle.getString("windowTitleNoDownload"));
+    document.title = fontPackageBundle.getString("windowTitleNoDownload");
     var cancelButton = document.getElementById("cancelButton");
     cancelButton.setAttribute("label", fontPackageBundle.getString("cancelButtonNoDownload"));
   } 
