@@ -1,3 +1,4 @@
+//@line 42 "/builds/tinderbox/Fx-Mozilla1.8.0/Linux_2.4.21-37.ELsmp_Depend/mozilla/toolkit/mozapps/downloads/content/downloads.js"
 
 ///////////////////////////////////////////////////////////////////////////////
 // Globals
@@ -19,7 +20,11 @@ var gActiveDownloads  = [];
 // opened by the xpinstall manager prevents the window from being closed after
 // each download completes (because xpinstall downloads are done sequentially, 
 // not concurrently) 
-var gCanAutoClose     = true;
+var gCanAutoClose   = true;
+
+// If the user has interacted with the window in a significant way, we should
+// not auto-close the window. Tough UI decisions about what is "significant."
+var gUserInteracted = false;
 
 ///////////////////////////////////////////////////////////////////////////////
 // Utility Functions 
@@ -90,16 +95,20 @@ function downloadCompleted(aDownload)
 
     var id = aDownload.targetFile.path;
     
-    // Refresh the icon, so that executable icons are shown.
-    var mimeService = Components.classes["@mozilla.org/uriloader/external-helper-app-service;1"].getService(Components.interfaces.nsIMIMEService);
-    var contentType = mimeService.getTypeFromFile(aDownload.targetFile);
-    
-    var listItem = document.getElementById(id);
-    var oldImage = listItem.getAttribute("image");
-    // I tack on the content-type here as a hack to bypass the cache which seems
-    // to be interfering despite the fact the image has 'validate="always"' set
-    // on it. 
-    listItem.setAttribute("image", oldImage + "&contentType=" + contentType);
+    // getTypeFromFile fails if it can't find a type for this file. Handle this gracefully.
+    try {
+      // Refresh the icon, so that executable icons are shown.
+      var mimeService = Components.classes["@mozilla.org/uriloader/external-helper-app-service;1"].getService(Components.interfaces.nsIMIMEService);
+      var contentType = mimeService.getTypeFromFile(aDownload.targetFile);
+
+      var listItem = document.getElementById(id);
+      var oldImage = listItem.getAttribute("image");
+      // I tack on the content-type here as a hack to bypass the cache which seems
+      // to be interfering despite the fact the image has 'validate="always"' set
+      // on it. 
+      listItem.setAttribute("image", oldImage + "&contentType=" + contentType);
+    } catch (e) {
+    }
     
     var dlRes = rdf.GetUnicodeResource(id);
   
@@ -126,7 +135,7 @@ function downloadCompleted(aDownload)
     gDownloadViewController.onCommandUpdate();
 
     if (gActiveDownloads.length == 0)
-      window.title = document.documentElement.getAttribute("statictitle");    
+      document.title = document.documentElement.getAttribute("statictitle");
   }
   catch (e) {
   }
@@ -138,14 +147,12 @@ function autoClose(aDownload)
     // For the moment, just use the simple heuristic that if this window was
     // opened by the download process, rather than by the user, it should auto-close
     // if the pref is set that way. If the user opened it themselves, it should
-    // not close until they explicitly close it. 
-    // We may like to revisit this in a bit more detail later, perhaps we want
-    // to keep it up if the user messes with it in a significant way.
+    // not close until they explicitly close it.
     var pref = Components.classes["@mozilla.org/preferences-service;1"]
                         .getService(Components.interfaces.nsIPrefBranch);
     var autoClose = pref.getBoolPref(PREF_BDM_CLOSEWHENDONE)
     if (autoClose && (!window.opener || window.opener.location.href == window.location.href) &&
-        gCanAutoClose)
+        gCanAutoClose && !gUserInteracted)
       gCloseDownloadManager();
   }
 }
@@ -173,9 +180,14 @@ var gDownloadObserver = {
       downloadCompleted(dl);
       break;
     case "dl-start":
+      // whenever a new download starts, it is added to the top, so switch the
+      // view to it
+      gDownloadsView.selectedIndex = 0;
+
       // Add this download to the percentage average tally
       var dl = aSubject.QueryInterface(Components.interfaces.nsIDownload);
       gActiveDownloads.push(dl);
+
       break;
     case "xpinstall-download-started":
       var windowArgs = aSubject.QueryInterface(Components.interfaces.nsISupportsArray);
@@ -201,6 +213,8 @@ var gDownloadObserver = {
 
 function onDownloadCancel(aEvent)
 {
+  var selectedIndex = gDownloadsView.selectedIndex;
+
   gDownloadManager.cancelDownload(aEvent.target.id);
 
   setRDFProperty(aEvent.target.id, "DownloadAnimated", "false");
@@ -217,26 +231,54 @@ function onDownloadCancel(aEvent)
     f.remove(false);
 
   gDownloadViewController.onCommandUpdate();
+
+  // now reset the richlistbox
+  gDownloadsView.clearSelection();
+  var rowCount = gDownloadsView.getRowCount();
+  if (selectedIndex >= rowCount)
+    gDownloadsView.selectedIndex = rowCount - 1;
+  else
+    gDownloadsView.selectedIndex = selectedIndex;
 }
 
 function onDownloadPause(aEvent)
 {
+  var selectedIndex = gDownloadsView.selectedIndex;
+
   var uri = aEvent.target.id;
   gDownloadManager.pauseDownload(uri);
   setRDFProperty(uri, "DownloadStatus", aEvent.target.getAttribute("status-internal"));
   setRDFProperty(uri, "ProgressPercent", aEvent.target.getAttribute("progress"));
+
+  // now reset the richlistbox
+  gDownloadsView.clearSelection();
+  var rowCount = gDownloadsView.getRowCount();
+  if (selectedIndex >= rowCount)
+    gDownloadsView.selectedIndex = rowCount - 1;
+  else
+    gDownloadsView.selectedIndex = selectedIndex;
 }
 
 function onDownloadResume(aEvent)
 {
+  var selectedIndex = gDownloadsView.selectedIndex;
+
   gDownloadManager.resumeDownload(aEvent.target.id);
+
+  // now reset the richlistbox
+  gDownloadsView.clearSelection();
+  var rowCount = gDownloadsView.getRowCount();
+  if (selectedIndex >= rowCount)
+    gDownloadsView.selectedIndex = rowCount - 1;
+  else
+    gDownloadsView.selectedIndex = selectedIndex;
 }
 
 function onDownloadRemove(aEvent)
 {
   if (aEvent.target.removable) {
+    var selectedIndex = gDownloadsView.selectedIndex;
     gDownloadManager.removeDownload(aEvent.target.id);
-    
     gDownloadViewController.onCommandUpdate();
   }
 }
@@ -250,11 +292,11 @@ function onDownloadShow(aEvent)
       f.reveal();
     } catch (ex) {
       // if reveal failed for some reason (eg on unix it's not currently
-      // implemented), send the file: URL  window rooted at the parent to 
+      // implemented), send the file: URL window rooted at the parent to 
       // the OS handler for that protocol
       var parent = f.parent;
       if (parent) {
-        openExternal(parent.path);
+        openExternal(parent);
       }
     }
   }
@@ -313,7 +355,7 @@ function onDownloadOpen(aEvent)
         } catch (ex) {
           // if launch fails, try sending it through the system's external
           // file: URL handler
-          openExternal(f.path);
+          openExternal(f);
         }
       }
       else {
@@ -343,6 +385,7 @@ function onDownloadOpenWith(aEvent)
 
 function onDownloadProperties(aEvent)
 {
+  gUserInteracted = true;
   window.openDialog("chrome://mozapps/content/downloads/downloadProperties.xul",
                     "_blank", "modal,centerscreen,chrome,resizable=no", aEvent.target.id);
 }
@@ -362,42 +405,54 @@ function onDownloadRetry(aEvent)
     var f = getLocalFileFromNativePathOrUrl(aEvent.target.id);
     saveURL(src, f, null, true, true);
   }
-  
+
   gDownloadViewController.onCommandUpdate();
+
+  // retry always places the item in the first spot
+  gDownloadsView.selectedIndex = 0;
 }
 
 // This is called by the progress listener. We don't actually use the event
 // system here to minimize time wastage. 
-var gLastComputedMean = 0;
+var gLastComputedMean = -1;
+var gLastActiveDownloads = 0;
 function onUpdateProgress()
 {
   var numActiveDownloads = gActiveDownloads.length;
   if (numActiveDownloads == 0) {
-    window.title = document.documentElement.getAttribute("statictitle");
-    gLastComputedMean = 0;
+    document.title = document.documentElement.getAttribute("statictitle");
+    gLastComputedMean = -1;
     return;
   }
     
   var mean = 0;
+  var base = 0;
+  var dl = null;
   for (var i = 0; i < numActiveDownloads; ++i) {
-    var dl = gActiveDownloads[i];
-    var progress = dl.percentComplete;
-    if (progress < 100)
-      mean += progress;
+    dl = gActiveDownloads[i];
+
+    // gActiveDownloads is screwed so it's possible 
+    // to have more files than we're really downloading.
+    // The good news is that those files have size==0.
+    // Same with files with unknown size. Their size==0.
+    if (dl.percentComplete < 100 && dl.size > 0) {
+      mean += dl.amountTransferred;
+      base += dl.size;
+    }
   }
 
-  mean = Math.round(mean / numActiveDownloads);
-  
-  // At the end of a download, progress is set from 100% to 0% for 
-  // some reason. We can identify this case because at this point the
-  // mean progress will be zero but the last computed mean will be
-  // greater than zero. 
-  if (mean == 0 && gLastComputedMean > 0) {
-    window.title = document.documentElement.getAttribute("statictitle");
-    return;
+  // we're not downloading anything at the moment,
+  // but we already downloaded something.
+  if (base == 0) {
+    mean = 100;
+  } else {
+    mean = Math.floor((mean / base) * 100);
   }
-  if (mean != gLastComputedMean) {
+
+  if (mean != gLastComputedMean || gLastActiveDownloads != numActiveDownloads) {
     gLastComputedMean = mean;
+    gLastActiveDownloads = numActiveDownloads;
+    
     var strings = document.getElementById("downloadStrings");
     
     var title;
@@ -406,7 +461,7 @@ function onUpdateProgress()
     else
       title = strings.getFormattedString("downloadsTitle", [mean]);
 
-    window.title = title;
+    document.title = title;
   }
 }
 
@@ -448,7 +503,7 @@ function Startup()
   // Set up AutoDownload display area
   initAutoDownloadDisplay();
   var pbi = Components.classes["@mozilla.org/preferences-service;1"]
-                      .getService(Components.interfaces.nsIPrefBranchInternal);
+                      .getService(Components.interfaces.nsIPrefBranch2);
   pbi.addObserver("browser.download.", gDownloadPrefObserver, false);
   
   // Handlers for events generated by the Download Manager (download events)
@@ -480,8 +535,9 @@ function Startup()
   // Finally, update the UI. 
   gDownloadsView.database.AddDataSource(gDownloadManager.datasource);
   gDownloadsView.builder.rebuild();
-  
-  gDownloadsView.focus();
+
+  // downloads can finish before Startup() does, so check if the window should close
+  autoClose();
 }
 
 function Shutdown() 
@@ -492,7 +548,7 @@ function Shutdown()
   gDownloadManager.saveState();
 
   var pbi = Components.classes["@mozilla.org/preferences-service;1"]
-                      .getService(Components.interfaces.nsIPrefBranchInternal);
+                      .getService(Components.interfaces.nsIPrefBranch2);
   pbi.removeObserver("browser.download.", gDownloadPrefObserver);
 
   var observerService = Components.classes[kObserverServiceProgID]
@@ -502,6 +558,7 @@ function Shutdown()
   observerService.removeObserver(gDownloadObserver, "dl-failed");  
   observerService.removeObserver(gDownloadObserver, "dl-start");  
   observerService.removeObserver(gDownloadObserver, "xpinstall-download-started");  
+  observerService.removeObserver(gDownloadObserver, "xpinstall-dialog-close");
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -516,18 +573,21 @@ var XPInstallDownloadManager = {
     var tempDir = fileLocator.get("TmpD", Components.interfaces.nsIFile);
 
     var mimeService = Components.classes["@mozilla.org/uriloader/external-helper-app-service;1"].getService(Components.interfaces.nsIMIMEService);
-    
+
+    var IOService = Components.classes["@mozilla.org/network/io-service;1"]
+                              .getService(Components.interfaces.nsIIOService);
+
     var xpinstallManager = gDownloadManager.QueryInterface(Components.interfaces.nsIXPInstallManagerUI);
 
     var xpiString = "";
+
     for (var i = 0; i < numXPInstallItems;) {
       // Pretty Name
       var displayName = aParams.GetString(i++);
       
       // URI
-      var uri = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURI);
-      uri.spec = aParams.GetString(i++);
-      
+      var uri = IOService.newURI(aParams.GetString(i++), null, null);
+
       var iconURL = aParams.GetString(i++);
       
       // Local File Target
@@ -547,7 +607,7 @@ var XPInstallDownloadManager = {
       if (!iconURL) 
         iconURL = "chrome://mozapps/skin/xpinstall/xpinstallItemGeneric.png";
       
-      var targetUrl = makeFileURL(localTarget);
+      var targetUrl = makeFileURI(localTarget);
       var download = gDownloadManager.addDownload(Components.interfaces.nsIXPInstallManagerUI.DOWNLOAD_TYPE_INSTALL, 
                                                   uri, targetUrl, displayName, iconURL, mimeInfo, 0, null);
       
@@ -574,14 +634,14 @@ var gContextMenus = [
 function buildContextMenu(aEvent)
 {
   if (aEvent.target.id != "downloadContextMenu")
-    return;
+    return false;
     
   var popup = document.getElementById("downloadContextMenu");
   while (popup.hasChildNodes())
     popup.removeChild(popup.firstChild);
   
-  if (gDownloadsView.selected) {
-    var idx = parseInt(gDownloadsView.selected.getAttribute("state"));
+  if (gDownloadsView.selectedItem) {
+    var idx = parseInt(gDownloadsView.selectedItem.getAttribute("state"));
     if (idx < 0)
       idx = 0;
     
@@ -674,8 +734,8 @@ function onDownloadShowOptions()
 
 function onDownloadShowInfo()
 {
-  if (gDownloadsView.selected)
-    fireEventForElement(gDownloadsView.selected, "properties");
+  if (gDownloadsView.selectedItem)
+    fireEventForElement(gDownloadsView.selectedItem, "properties");
 }
 
 function initAutoDownloadDisplay()
@@ -692,6 +752,7 @@ function initAutoDownloadDisplay()
 
     function getSpecialFolderKey(aFolderType) 
     {
+//@line 807 "/builds/tinderbox/Fx-Mozilla1.8.0/Linux_2.4.21-37.ELsmp_Depend/mozilla/toolkit/mozapps/downloads/content/downloads.js"
       return "Home";
     }
     
@@ -713,11 +774,11 @@ function initAutoDownloadDisplay()
     }
 
     var displayName = null;
+    var folder;
     switch (pref.getIntPref("browser.download.folderList")) {
     case 0:
       folder = getDownloadsFolder("Desktop");
-      var strings = document.getElementById("downloadStrings");
-      displayName = strings.getString("displayNameDesktop");
+      displayName = document.getElementById("downloadStrings").getString("displayNameDesktop");
       break;
     case 1:
       folder = getDownloadsFolder("Downloads");
@@ -774,19 +835,20 @@ function onDownloadShowFolder()
   } catch (ex) {
     // if nsILocalFile::Reveal failed (eg it currently just returns an
     // error on unix), just open the folder in a browser window
-    openExternal(dir.path);
+    openExternal(dir);
   }
 }
 
-function openExternal(aPath)
+function openExternal(aFile)
 {
-  var uri = Components.classes["@mozilla.org/network/standard-url;1"]
-    .createInstance(Components.interfaces.nsIURI);
-  uri.spec = "file:///" + aPath;
+  var uri = Components.classes["@mozilla.org/network/io-service;1"]
+                      .getService(Components.interfaces.nsIIOService)
+                      .newFileURI(aFile);
+
+  var protocolSvc = 
+      Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
+                .getService(Components.interfaces.nsIExternalProtocolService);
 
-  var protocolSvc = Components.classes
-    ["@mozilla.org/uriloader/external-protocol-service;1"]
-    .getService(Components.interfaces.nsIExternalProtocolService);
   protocolSvc.loadUrl(uri);
 
   return;
@@ -811,7 +873,7 @@ function getLocalFileFromNativePathOrUrl(aPathOrUrl)
   } else {
 
     // if it's a pathname, create the nsILocalFile directly
-    f = Components.classes["@mozilla.org/file/local;1"].
+    var f = Components.classes["@mozilla.org/file/local;1"].
       createInstance(Components.interfaces.nsILocalFile);
     f.initWithPath(aPathOrUrl);
 
